@@ -3,7 +3,7 @@
 /*
  *  File-Roller
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,6 +28,9 @@
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 
 
+#define FILENAME_MAX_LENGTH 30 /* FIXME: find out the best value */
+
+
 gboolean            path_is_file                 (const gchar *s);
 
 gboolean            path_is_dir                  (const gchar *s);
@@ -80,4 +83,17 @@ gboolean            match_patterns               (char       **patterns,
 
 char **             search_util_get_patterns     (const char  *pattern_string);
 
+GnomeVFSFileSize    get_dest_free_space          (const char  *path);
+
+gboolean            rmdir_recursive              (const gchar *directory);
+
+char *              _g_strdup_with_max_size      (const char *s,
+						  int         max_size);
+
+#define get_home_relative_dir(x)        \
+        g_strconcat (g_get_home_dir (), \
+                     "/",               \
+                     (x),               \
+                     NULL)
+
 #endif /* FILE_UTILS_H */
