@@ -3,7 +3,7 @@
 /*
  *  File-Roller
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@
 #include "fr-command-rar.h"
 #include "fr-command-tar.h"
 #include "fr-command-zip.h"
+#include "fr-command-zoo.h"
 #include "fr-marshal.h"
 #include "fr-process.h"
 
@@ -45,6 +46,9 @@
 enum {
 	START,
 	DONE,
+	PROGRESS,
+	MESSAGE,
+	STOPPABLE,
 	LAST_SIGNAL
 };
 
@@ -110,10 +114,59 @@ fr_archive_class_init (FRArchiveClass *class)
 			      G_TYPE_NONE, 2,
 			      G_TYPE_INT,
 			      G_TYPE_POINTER);
+	fr_archive_signals[PROGRESS] =
+		g_signal_new ("progress",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (FRArchiveClass, progress),
+			      NULL, NULL,
+			      fr_marshal_VOID__DOUBLE,
+			      G_TYPE_NONE, 1,
+			      G_TYPE_DOUBLE);
+	fr_archive_signals[MESSAGE] =
+		g_signal_new ("message",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (FRArchiveClass, message),
+			      NULL, NULL,
+			      fr_marshal_VOID__STRING,
+			      G_TYPE_NONE, 1,
+			      G_TYPE_STRING);
+	fr_archive_signals[STOPPABLE] =
+                g_signal_new ("stoppable",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (FRArchiveClass, stoppable),
+			      NULL, NULL,
+			      fr_marshal_VOID__BOOL,
+			      G_TYPE_NONE, 
+			      1, G_TYPE_BOOLEAN);
 	
 	gobject_class->finalize = fr_archive_finalize;
 	class->start = NULL;
 	class->done = NULL;
+	class->progress = NULL;
+	class->message = NULL;
+}
+
+
+void
+fr_archive_stoppable (FRArchive *archive,
+		      gboolean   stoppable)
+{
+	g_signal_emit (G_OBJECT (archive), 
+		       fr_archive_signals[STOPPABLE],
+		       0,
+		       stoppable);
+}
+
+
+static gboolean
+archive_sticky_only_cb (FRProcess *process,
+			FRArchive *archive)
+{
+	fr_archive_stoppable (archive, FALSE);
+	return TRUE;
 }
 
 
@@ -122,14 +175,21 @@ fr_archive_init (FRArchive *archive)
 {
 	archive->filename = NULL;
 	archive->command = NULL;
-	archive->process = fr_process_new ();
 	archive->is_compressed_file = FALSE;
-	archive->fake_load = FALSE;
+
+	archive->fake_load_func = NULL;
+	archive->fake_load_data = NULL;
+
+	archive->process = fr_process_new ();
+	g_signal_connect (G_OBJECT (archive->process), 
+			  "sticky_only",
+			  G_CALLBACK (archive_sticky_only_cb),
+			  archive);
 }
 
 
 FRArchive *
-fr_archive_new ()
+fr_archive_new (void)
 {
 	return FR_ARCHIVE (g_object_new (FR_TYPE_ARCHIVE, NULL));
 }
@@ -183,6 +243,9 @@ create_command_from_mime_type (FRArchive  *archive,
 	} else if (is_mime_type (mime_type, "application/zip")) {
 		archive->command = fr_command_zip_new (archive->process, 
 						       filename);
+	} else if (is_mime_type (mime_type, "application/x-zoo")) {
+		archive->command = fr_command_zoo_new (archive->process,
+						       filename);
 	} else if (is_mime_type (mime_type, "application/x-rar")) {
 		archive->command = fr_command_rar_new (archive->process, 
 						       filename);
@@ -271,8 +334,6 @@ create_command_from_filename (FRArchive  *archive,
 			      const char *filename,
 			      gboolean    loading)
 {
-	archive->is_compressed_file = FALSE;
-
 	if (file_extension_is (filename, ".tar.gz")
 	    || file_extension_is (filename, ".tgz")) {
 		archive->command = fr_command_tar_new (archive->process, 
@@ -308,6 +369,9 @@ create_command_from_filename (FRArchive  *archive,
 		   || file_extension_is (filename, ".war")) {
 		archive->command = fr_command_zip_new (archive->process, 
 						       filename);
+	} else if (file_extension_is (filename, ".zoo")) {
+		archive->command = fr_command_zoo_new (archive->process,
+						       filename);
 	} else if (file_extension_is (filename, ".lzh")) {
 		archive->command = fr_command_lha_new (archive->process, 
 						       filename);
@@ -375,6 +439,9 @@ action_performed (FRCommand   *command,
 	case FR_ACTION_TEST:
 		s_action = "Test";
 		break;
+	case FR_ACTION_GET_LIST:
+		s_action = "Get list";
+		break;
 	}
 	g_print ("%s [DONE]\n", s_action);
 #endif
@@ -387,6 +454,32 @@ action_performed (FRCommand   *command,
 }
 
 
+static gboolean
+archive_progress_cb (FRCommand  *command,
+		     double      fraction,
+		     FRArchive  *archive)
+{
+	g_signal_emit (G_OBJECT (archive), 
+		       fr_archive_signals[PROGRESS], 
+		       0,
+		       fraction);
+	return TRUE;
+}
+
+
+static gboolean
+archive_message_cb  (FRCommand  *command,
+		     const char *msg,
+		     FRArchive  *archive)		     
+{
+	g_signal_emit (G_OBJECT (archive), 
+		       fr_archive_signals[MESSAGE], 
+		       0,
+		       msg);
+	return TRUE;
+}
+
+
 /* filename must not be escaped. */
 gboolean
 fr_archive_new_file (FRArchive  *archive, 
@@ -420,11 +513,39 @@ fr_archive_new_file (FRArchive  *archive,
 			  "done",
 			  G_CALLBACK (action_performed),
 			  archive);
+	g_signal_connect (G_OBJECT (archive->command), 
+			  "progress",
+			  G_CALLBACK (archive_progress_cb),
+			  archive);
+	g_signal_connect (G_OBJECT (archive->command), 
+			  "message",
+			  G_CALLBACK (archive_message_cb),
+			  archive);
 
 	return TRUE;
 }
 
 
+void
+fr_archive_set_fake_load_func (FRArchive    *archive,
+			       FakeLoadFunc  func,
+			       gpointer      data)
+{
+	archive->fake_load_func = func;
+	archive->fake_load_data = data;
+}
+
+
+gboolean
+fr_archive_fake_load (FRArchive *archive)
+{
+	if (archive->fake_load_func != NULL)
+		return (*archive->fake_load_func) (archive, archive->fake_load_data);
+	else
+		return FALSE;
+}
+
+
 /* filename must not be escaped. */
 gboolean
 fr_archive_load (FRArchive  *archive, 
@@ -466,13 +587,21 @@ fr_archive_load (FRArchive  *archive,
 			  "start",
 			  G_CALLBACK (action_started),
 			  archive);
-
 	g_signal_connect (G_OBJECT (archive->command), 
 			  "done",
 			  G_CALLBACK (action_performed),
 			  archive);
+	g_signal_connect (G_OBJECT (archive->command), 
+			  "progress",
+			  G_CALLBACK (archive_progress_cb),
+			  archive);
+	g_signal_connect (G_OBJECT (archive->command), 
+			  "message",
+			  G_CALLBACK (archive_message_cb),
+			  archive);
 
-	archive->command->fake_load = archive->fake_load;
+	fr_archive_stoppable (archive, TRUE);
+	archive->command->fake_load = fr_archive_fake_load (archive);
 	fr_command_list (archive->command);
 
 	return TRUE;
@@ -485,7 +614,8 @@ fr_archive_reload (FRArchive *archive)
 	g_return_if_fail (archive != NULL);
 	g_return_if_fail (archive->filename != NULL);
 
-	archive->command->fake_load = archive->fake_load;
+	fr_archive_stoppable (archive, TRUE);
+	archive->command->fake_load = fr_archive_fake_load (archive);
 	fr_command_list (archive->command);
 }
 
@@ -502,6 +632,7 @@ fr_archive_rename (FRArchive  *archive,
 		 * because in this case the 'content' of the archive changes 
 		 * too. */
 		fr_archive_load (archive, filename);
+
 	else {
 		if (archive->filename != NULL)
 			g_free (archive->filename);
@@ -524,7 +655,6 @@ find_file_in_archive (FRArchive *archive,
 
 	for (scan = archive->command->file_list; scan; scan = scan->next) {
 		FileData *fdata = scan->data;
-		
 		if (strcmp (path, fdata->original_path) == 0)
 			return fdata;
 	}
@@ -562,8 +692,8 @@ newer_files_only (FRArchive  *archive,
 	GList *scan;
 
 	for (scan = file_list; scan; scan = scan->next) {
-		gchar *filename = scan->data;
-		gchar *fullpath;
+		char     *filename = scan->data;
+		char     *fullpath;
 		FileData *fdata;
 
 		fdata = find_file_in_archive (archive, filename);
@@ -591,11 +721,11 @@ newer_files_only (FRArchive  *archive,
 
 /* Note: all paths unescaped. */
 void
-fr_archive_add (FRArchive   *archive, 
-		GList       *file_list, 
-		const char  *base_dir,
-		gboolean     update,
-		const char  *password,
+fr_archive_add (FRArchive     *archive, 
+		GList         *file_list, 
+		const char    *base_dir,
+		gboolean       update,
+		const char    *password,
 		FRCompression  compression)
 {
 	GList    *new_file_list;
@@ -609,6 +739,8 @@ fr_archive_add (FRArchive   *archive,
 	if (archive->read_only)
 		return;
 
+	fr_archive_stoppable (archive, FALSE);
+
 	/* if the command cannot update,  get the list of files that are 
 	 * newer than the ones in the archive. */
 
@@ -623,6 +755,12 @@ fr_archive_add (FRArchive   *archive,
 #ifdef DEBUG
 		g_print ("nothing to update.\n");
 #endif
+
+		archive->process->error.type = FR_PROC_ERROR_NONE;
+		g_signal_emit_by_name (G_OBJECT (archive->process), 
+				       "done",
+				       FR_ACTION_ADD,
+				       &archive->process->error);
 		return;
 	}
 
@@ -642,7 +780,7 @@ fr_archive_add (FRArchive   *archive,
 		GList *del_list = NULL;
 
 		for (scan = new_file_list; scan != NULL; scan = scan->next) {
-			gchar *filename = scan->data;
+			char *filename = scan->data;
 			if (find_file_in_archive (archive, filename)) 
 				del_list = g_list_prepend (del_list, filename);
 		}
@@ -658,6 +796,7 @@ fr_archive_add (FRArchive   *archive,
 	/* add now. */
 
 	e_file_list = escape_file_list (new_file_list);
+	fr_command_set_n_files (archive->command, g_list_length (e_file_list));
 
 	for (scan = e_file_list; scan != NULL; ) {
 		GList *prev = scan->prev;
@@ -690,7 +829,6 @@ fr_archive_add (FRArchive   *archive,
 		g_list_free (new_file_list);
 
 	fr_command_recompress (archive->command, compression);
-
 	fr_process_start (archive->process);
 }
 
@@ -723,71 +861,166 @@ file_list_remove_from_pattern (GList      **list,
 }
 
 
-/* Note: all paths unescaped. */
-void
-fr_archive_add_with_wildcard (FRArchive  *archive, 
-			      const char *include_files,
-			      const char *exclude_files,
-			      const char *base_dir,
-			      gboolean    update,
-			      gboolean    recursive,
-			      gboolean    follow_links,
-			      gboolean    same_fs,
-			      gboolean    no_backup_files,
-			      gboolean    no_dot_files,
-			      gboolean    ignore_case,
-			      const char *password,
-			      FRCompression compression)
+/* -- add with wildcard -- */
+
+
+typedef struct {
+	FRArchive     *archive;
+        char          *exclude_files;
+	char          *base_dir;
+	gboolean       update;
+	char          *password;
+	FRCompression  compression;
+	DoneFunc       done_func;
+	gpointer       done_data;
+} AddWithWildcardData;
+
+
+static void
+add_with_wildcard__step2 (GList *file_list, gpointer data)
 {
-	GList *file_list;
+	AddWithWildcardData *aww_data = data;
 
-	if (archive->read_only)
-		return;
+	file_list_remove_from_pattern (&file_list, aww_data->exclude_files);
 
-	file_list = get_wildcard_file_list (base_dir, 
-					    include_files, 
-					    recursive, 
-					    follow_links, 
-					    same_fs,
-					    no_backup_files, 
-					    no_dot_files, 
-					    ignore_case);
+	if (aww_data->done_func) 
+		aww_data->done_func (aww_data->done_data);
 
-	file_list_remove_from_pattern (&file_list, exclude_files);
-
-	fr_archive_add (archive,
+	fr_archive_add (aww_data->archive,
 			file_list,
-			base_dir,
-			update,
-			password,
-			compression);
+			aww_data->base_dir,
+			aww_data->update,
+			aww_data->password,
+			aww_data->compression);
 	path_list_free (file_list);
+
+	g_free (aww_data->base_dir);
+	g_free (aww_data->password);
+	g_free (aww_data->exclude_files);
 }
 
 
 /* Note: all paths unescaped. */
-void
-fr_archive_add_directory (FRArchive  *archive, 
-			  const char *directory,
-			  const char *base_dir,
-			  gboolean    update,
-			  const char *password,
-			  FRCompression compression)
-
+VisitDirHandle *
+fr_archive_add_with_wildcard (FRArchive     *archive, 
+			      const char    *include_files,
+			      const char    *exclude_files,
+			      const char    *base_dir,
+			      gboolean       update,
+			      gboolean       recursive,
+			      gboolean       follow_links,
+			      gboolean       same_fs,
+			      gboolean       no_backup_files,
+			      gboolean       no_dot_files,
+			      gboolean       ignore_case,
+			      const char    *password,
+			      FRCompression  compression,
+			      DoneFunc       done_func,
+			      gpointer       done_data)
 {
-	GList *file_list;
+	AddWithWildcardData *aww_data;
 
 	if (archive->read_only)
-		return;
+		return NULL;
 
-	file_list = get_directory_file_list (directory, base_dir);
-	fr_archive_add (archive,
+	fr_archive_stoppable (archive, TRUE);
+
+	aww_data = g_new0 (AddWithWildcardData, 1);
+	aww_data->archive = archive;
+	aww_data->base_dir = g_strdup (base_dir);
+	aww_data->update = update;
+	aww_data->password = g_strdup (password);
+	aww_data->compression = compression;
+	aww_data->exclude_files = g_strdup (exclude_files);
+	aww_data->done_func = done_func;
+	aww_data->done_data = done_data;
+
+	return get_wildcard_file_list_async (base_dir, 
+					     include_files, 
+					     recursive, 
+					     follow_links, 
+					     same_fs,
+					     no_backup_files, 
+					     no_dot_files, 
+					     ignore_case,
+					     add_with_wildcard__step2, 
+					     aww_data);
+}
+
+
+/* -- fr_archive_add_directory -- */
+
+
+typedef struct {
+	FRArchive     *archive;
+	char          *directory;
+	char          *base_dir;
+	gboolean       update;
+	char          *password;
+	FRCompression  compression;
+	DoneFunc       done_func;
+	gpointer       done_data;
+} AddDirectoryData;
+
+
+static void
+add_directory__step2 (GList *file_list, gpointer data)
+{
+	AddDirectoryData *ad_data = data;
+
+	if (ad_data->done_func) 
+		ad_data->done_func (ad_data->done_data);
+
+	fr_archive_add (ad_data->archive,
 			file_list,
-			base_dir,
-			update,
-			password,
-			compression);
+			ad_data->base_dir,
+			ad_data->update,
+			ad_data->password,
+			ad_data->compression);
 	path_list_free (file_list);
+
+	/**/
+
+	g_free (ad_data->directory);
+	g_free (ad_data->base_dir);
+	g_free (ad_data->password);
+	g_free (ad_data);
+}
+
+
+/* Note: all paths unescaped. */
+VisitDirHandle *
+fr_archive_add_directory (FRArchive     *archive, 
+			  const char    *directory,
+			  const char    *base_dir,
+			  gboolean       update,
+			  const char    *password,
+			  FRCompression  compression,
+			  DoneFunc       done_func,
+			  gpointer       done_data)
+
+{
+	AddDirectoryData *ad_data;
+
+	if (archive->read_only)
+		return NULL;
+
+	fr_archive_stoppable (archive, TRUE);
+
+	ad_data = g_new0 (AddDirectoryData, 1);
+	ad_data->archive = archive;
+	ad_data->directory = g_strdup (directory);
+	ad_data->base_dir = g_strdup (base_dir);
+	ad_data->update = update;
+	ad_data->password = g_strdup (password);
+	ad_data->compression = compression;
+	ad_data->done_func = done_func;
+	ad_data->done_data = done_data;
+
+	return get_directory_file_list_async (directory, 
+					      base_dir, 
+					      add_directory__step2, 
+					      ad_data);
 }
 
 
@@ -808,6 +1041,7 @@ _archive_remove (FRArchive *archive,
 		file_list = archive->command->file_list;
 
 	e_file_list = escape_file_list (file_list);
+	fr_command_set_n_files (archive->command, g_list_length (e_file_list));
 
 	for (scan = e_file_list; scan != NULL; ) {
 		GList *prev = scan->prev;
@@ -845,6 +1079,8 @@ fr_archive_remove (FRArchive     *archive,
 	if (archive->read_only)
 		return;
 
+	fr_archive_stoppable (archive, FALSE);
+
 	fr_process_clear (archive->process);
 	fr_command_uncompress (archive->command);
 	_archive_remove (archive, file_list);
@@ -940,6 +1176,8 @@ extract_in_chunks (FRCommand  *command,
 {
 	GList *scan;
 
+	fr_command_set_n_files (command, g_list_length (file_list));
+
 	if (file_list == NULL) {
 		fr_command_extract (command,
 				    file_list,
@@ -999,6 +1237,8 @@ fr_archive_extract (FRArchive  *archive,
 
 	g_return_if_fail (archive != NULL);
 
+	fr_archive_stoppable (archive, TRUE);
+
 	/* if a command supports all the requested options use 
 	 * fr_command_extract directly. */
 
@@ -1159,7 +1399,10 @@ void
 fr_archive_test (FRArchive  *archive,
 		 const char *password)
 {
+	fr_archive_stoppable (archive, TRUE);
+
 	fr_process_clear (archive->process);
+	fr_command_set_n_files (archive->command, 0);
 	fr_command_test (archive->command, password);
 	fr_process_start (archive->process);
 }
@@ -1196,6 +1439,7 @@ fr_archive_utils__get_file_name_ext (const char *filename)
 		".war",
 		".z", 
 		".zip",
+		".zoo",
 		".Z" 
 	};
 	int n = sizeof (ext) / sizeof (char*);
