@@ -3,7 +3,7 @@
 /*
  *  File-Roller
  *
- *  Copyright (C) 2001 The Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
@@ -27,6 +29,7 @@
 #include <glib.h>
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
+#include <gnome.h>
 
 #include "file-data.h"
 #include "file-utils.h"
@@ -176,7 +179,7 @@ process_line (char     *line,
 
 	if (*fdata->name == 0)
 		file_data_free (fdata);
-	else
+	else 
 		comm->file_list = g_list_prepend (comm->file_list, fdata);
 }
 
@@ -233,6 +236,44 @@ fr_command_tar_list (FRCommand *comm)
 }
 
 
+static void
+process_line__generic (char     *line, 
+		       gpointer  data,
+		       char     *action_msg)
+{
+	FRCommand    *comm = FR_COMMAND (data);
+	char         *msg;
+	char         *filename;
+	double        fraction;
+
+	if (line == NULL)
+		return;
+
+	if (line[strlen (line) - 1] == '/') /* ignore directories */
+		return;
+
+	filename = _g_strdup_with_max_size (line, FILENAME_MAX_LENGTH);
+	msg = g_strconcat (action_msg, filename, NULL);
+	g_free (filename);
+
+	fr_command_message (comm, msg);
+	g_free (msg);
+
+	if (comm->n_files != 0) {
+		fraction = (double) comm->n_file++ / comm->n_files;
+		fr_command_progress (comm, fraction);
+	}
+}
+
+
+static void
+process_line__add (char     *line, 
+		   gpointer  data)
+{
+	process_line__generic (line, data, _("Adding file: "));
+}
+
+
 static void
 fr_command_tar_add (FRCommand     *comm,
 		    GList         *file_list,
@@ -244,11 +285,13 @@ fr_command_tar_add (FRCommand     *comm,
 	FRCommandTar *c_tar = FR_COMMAND_TAR (comm);
 	GList        *scan;
 
-	/* Add files. */
+	fr_process_set_out_line_func (FR_COMMAND (comm)->process, 
+				      process_line__add,
+				      comm);
 
 	fr_process_begin_command (comm->process, "tar");
-
 	fr_process_add_arg (comm->process, "--force-local");
+	fr_process_add_arg (comm->process, "-v");
 
 	if (base_dir != NULL) {
 		gchar *e_base_dir = shell_escape (base_dir);
@@ -269,6 +312,23 @@ fr_command_tar_add (FRCommand     *comm,
 }
 
 
+static void
+process_line__delete (char     *line, 
+		      gpointer  data)
+{
+	process_line__generic (line, data, _("Removing file: "));
+}
+
+
+static void
+begin_func__delete (gpointer data)
+{
+	FRCommand *comm = data;
+	fr_command_progress (comm, -1.0);
+	fr_command_message (comm, _("Deleting files from archive"));
+}
+
+
 static void
 fr_command_tar_delete (FRCommand *comm,
 		       GList     *file_list)
@@ -276,10 +336,14 @@ fr_command_tar_delete (FRCommand *comm,
 	FRCommandTar *c_tar = FR_COMMAND_TAR (comm);
 	GList        *scan;
 
-	/* Delete files. */
+	fr_process_set_out_line_func (comm->process, 
+				      process_line__delete,
+				      comm);
 
 	fr_process_begin_command (comm->process, "tar");
+	fr_process_set_begin_func (comm->process, begin_func__delete, comm);
 	fr_process_add_arg (comm->process, "--force-local");
+	fr_process_add_arg (comm->process, "-v");
 	fr_process_add_arg (comm->process, "--delete");
 	fr_process_add_arg (comm->process, "-f");
 	fr_process_add_arg (comm->process, c_tar->uncomp_filename);
@@ -290,6 +354,14 @@ fr_command_tar_delete (FRCommand *comm,
 }
 
 
+static void
+process_line__extract (char     *line, 
+		       gpointer  data)
+{
+	process_line__generic (line, data, _("Extracting file: "));
+}
+
+
 static void
 fr_command_tar_extract (FRCommand  *comm,
 			GList      *file_list,
@@ -301,16 +373,19 @@ fr_command_tar_extract (FRCommand  *comm,
 {
 	GList *scan;
 
-	fr_process_begin_command (comm->process, "tar");
-	
-	fr_process_add_arg (comm->process, "--force-local");
+	fr_process_set_out_line_func (FR_COMMAND (comm)->process, 
+				      process_line__extract,
+				      comm);
 
+	fr_process_begin_command (comm->process, "tar");
+	fr_process_add_arg (comm->process, "--force-local");
+	fr_process_add_arg (comm->process, "-v");
 	fr_process_add_arg (comm->process, "-xf");
 	fr_process_add_arg (comm->process, comm->e_filename);
 	add_compress_arg (comm);
 
 	if (dest_dir != NULL) {
-		gchar *e_dest_dir = shell_escape (dest_dir);
+		char *e_dest_dir = shell_escape (dest_dir);
 		fr_process_add_arg (comm->process, "-C");
 		fr_process_add_arg (comm->process, e_dest_dir);
 		g_free (e_dest_dir);
@@ -323,12 +398,21 @@ fr_command_tar_extract (FRCommand  *comm,
 }
 
 
+static void
+begin_func__recompress (gpointer data)
+{
+	FRCommand *comm = data;
+	fr_command_progress (comm, -1.0);
+	fr_command_message (comm, _("Recompressing archive"));
+}
+
+
 static void
 fr_command_tar_recompress (FRCommand     *comm,
 			   FRCompression  compression)
 {
 	FRCommandTar *c_tar = FR_COMMAND_TAR (comm);
-	gchar        *new_name = NULL;
+	char         *new_name = NULL;
 
 	switch (c_tar->compress_prog) {
 	case FR_COMPRESS_PROGRAM_NONE:
@@ -337,6 +421,7 @@ fr_command_tar_recompress (FRCommand     *comm,
 	case FR_COMPRESS_PROGRAM_GZIP:
 		fr_process_begin_command (comm->process, "gzip");
 		fr_process_set_sticky (comm->process, TRUE);
+		fr_process_set_begin_func (comm->process, begin_func__recompress, comm);
 		switch (compression) {
 		case FR_COMPRESSION_VERY_FAST:
 			fr_process_add_arg (comm->process, "-1"); break;
@@ -357,6 +442,7 @@ fr_command_tar_recompress (FRCommand     *comm,
 	case FR_COMPRESS_PROGRAM_BZIP:
 		fr_process_begin_command (comm->process, "bzip");
 		fr_process_set_sticky (comm->process, TRUE);
+		fr_process_set_begin_func (comm->process, begin_func__recompress, comm);
 		switch (compression) {
 		case FR_COMPRESSION_VERY_FAST:
 			fr_process_add_arg (comm->process, "-1"); break;
@@ -377,6 +463,7 @@ fr_command_tar_recompress (FRCommand     *comm,
 	case FR_COMPRESS_PROGRAM_BZIP2:
 		fr_process_begin_command (comm->process, "bzip2");
 		fr_process_set_sticky (comm->process, TRUE);
+		fr_process_set_begin_func (comm->process, begin_func__recompress, comm);
 		switch (compression) {
 		case FR_COMPRESSION_VERY_FAST:
 			fr_process_add_arg (comm->process, "-1"); break;
@@ -397,6 +484,7 @@ fr_command_tar_recompress (FRCommand     *comm,
 	case FR_COMPRESS_PROGRAM_COMPRESS: 
 		fr_process_begin_command (comm->process, "compress");
 		fr_process_set_sticky (comm->process, TRUE);
+		fr_process_set_begin_func (comm->process, begin_func__recompress, comm);
 		fr_process_add_arg (comm->process, "-f");
 		fr_process_add_arg (comm->process, c_tar->uncomp_filename);
 		fr_process_end_command (comm->process);
@@ -407,6 +495,7 @@ fr_command_tar_recompress (FRCommand     *comm,
 	case FR_COMPRESS_PROGRAM_LZOP:
 		fr_process_begin_command (comm->process, "lzop");
 		fr_process_set_sticky (comm->process, TRUE);
+		fr_process_set_begin_func (comm->process, begin_func__recompress, comm);
 		switch (compression) {
 		case FR_COMPRESSION_VERY_FAST:
 			fr_process_add_arg (comm->process, "-1"); break;
@@ -443,6 +532,15 @@ fr_command_tar_recompress (FRCommand     *comm,
 }
 
 
+static void
+begin_func__uncompress (gpointer data)
+{
+	FRCommand *comm = data;
+	fr_command_progress (comm, -1.0);
+	fr_command_message (comm, _("Decompressing archive"));
+}
+
+
 static void
 fr_command_tar_uncompress (FRCommand *comm)
 {
@@ -466,6 +564,7 @@ fr_command_tar_uncompress (FRCommand *comm)
 	case FR_COMPRESS_PROGRAM_GZIP:
 		if (path_is_file (comm->filename)) {
 			fr_process_begin_command (comm->process, "gzip");
+			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
 			fr_process_add_arg (comm->process, "-f");
 			fr_process_add_arg (comm->process, "-d");
 			fr_process_add_arg (comm->process, comm->e_filename);
@@ -485,6 +584,7 @@ fr_command_tar_uncompress (FRCommand *comm)
 	case FR_COMPRESS_PROGRAM_BZIP:
 		if (path_is_file (comm->filename)) {
 			fr_process_begin_command (comm->process, "bzip");
+			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
 			fr_process_add_arg (comm->process, "-f");
 			fr_process_add_arg (comm->process, "-d");
 			fr_process_add_arg (comm->process, comm->e_filename);
@@ -504,6 +604,7 @@ fr_command_tar_uncompress (FRCommand *comm)
 	case FR_COMPRESS_PROGRAM_BZIP2:
 		if (path_is_file (comm->filename)) {
 			fr_process_begin_command (comm->process, "bzip2");
+			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
 			fr_process_add_arg (comm->process, "-f");
 			fr_process_add_arg (comm->process, "-d");
 			fr_process_add_arg (comm->process, comm->e_filename);
@@ -524,6 +625,7 @@ fr_command_tar_uncompress (FRCommand *comm)
 	case FR_COMPRESS_PROGRAM_COMPRESS: 
 		if (path_is_file (comm->filename)) {
 			fr_process_begin_command (comm->process, "uncompress");
+			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
 			fr_process_add_arg (comm->process, "-f");
 			fr_process_add_arg (comm->process, comm->e_filename);
 			fr_process_end_command (comm->process);
@@ -541,6 +643,7 @@ fr_command_tar_uncompress (FRCommand *comm)
 	case FR_COMPRESS_PROGRAM_LZOP:
 		if (path_is_file (comm->filename)) {
 			fr_process_begin_command (comm->process, "lzop");
+			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
 			fr_process_add_arg (comm->process, "-dfU");
 			fr_process_add_arg (comm->process, "--no-stdin");
 			fr_process_add_arg (comm->process, comm->e_filename);
